#ifndef _TESTCASE_H_
#define _TESTCASE_H_

#include <string>
#include <vector>
#include <fstream>

class FailedTestException
{
 public:
  FailedTestException(std::string reason) : m_reason(reason) {}
  std::string m_reason;
};

class TestResult
{
 public:
  TestResult(std::string classname, std::string method)
    : m_classname(classname), m_method(method), m_passed(true),
    m_text("Passed") 
    {}

  std::string m_classname;
  std::string m_method;
  bool m_passed;
  std::string m_text;
};

typedef std::vector<TestResult> TestResults;

class TestCase
{
  // You should derive from this class, providing an implementation
  // of run() with the prototype below and implemented by using the
  // macros BEGIN_TESTS, TEST and END_TESTS.
  // Also optionally override setUp and tearDown()

 public:
  // static TestResults run()
  virtual void setUp() {}
  virtual void tearDown() {}

 protected:
  void assertImplementation(bool condition, std::string text);
  static void checkOutput(std::string tmp, std::string expected);
};

#undef assert
#define assert(condition)\
    (this->assertImplementation ((condition),(#condition)))

#define DEFINE_TESTS() static TestResults run(std::string testname);


#define BEGIN_TESTS(class)  \
TestResults class::run(std::string testname) \
{ \
   TestResults results; 

#define TEST(class, method) \
   { \
    if (testname.size() == 0 || testname == (#class"::"#method)) { \
      TestResult r(#class, #method); \
      try { \
       class c; \
       c.setUp(); \
       c.method(); \
       c.tearDown(); \
      } catch(FailedTestException e) { \
       r.m_passed = false; \
       r.m_text = e.m_reason; \
      } \
      catch (...) { \
        r.m_passed = false; \
        r.m_text = "Unhandled exception occured"; \
      }\
      results.push_back(r);\
     } \
   }

#define TEST_WITH_OUTPUT(class, method) \
   { \
    if (testname.size() == 0 || testname == #class"::"#method) { \
      TestResult r(#class, #method); \
      try { \
       class c; \
       ofstream out; \
       out.open("/tmp/run-"#class"-"#method".txt"); \
       c.setUp(); \
       c.method(out); \
       c.tearDown(); \
       out.close(); \
       TestCase::checkOutput("/tmp/run-"#class"-"#method".txt", "expected-"#class"-"#method".txt");\
      } catch(FailedTestException e) { \
      r.m_passed = false; \
      r.m_text = e.m_reason; \
      } \
      catch (...) { \
        r.m_passed = false; \
        r.m_text = "Unhandled exception occured"; \
      }\
    results.push_back(r);\
   } \
  }

#define END_TESTS() \
  return results; \
 }


// 'results' is a variable of type 'TestResults'.  The results of the test being
//    run will be added to it.
// 'class' is the testcase which you wish to run.
// 'test' is the name (in the form "class::method") of the test which you 
//    wish to run.  It this is the empty string, all tests will be run.
#define RUN_TESTS(results, class, test) { TestResults r = class::run(test); \
    results.insert(results.begin(), r.begin(), r.end()); }

#endif // _TESTCASE_H_
